from typing import List

import ffmpeg

from lecture_automator.gen_video.utils import get_codecs


def generate_video(path_images: List[str], path_wavs: List[str],
                   output_name: str, vformat: str='mp4') -> None:
    """Генерация видео на основе изображений и звука к каждому из них.

    Args:
        path_images (List[str]): список путей к изображениям в файловой системе.
        path_wavs (List[str]): список путей к звукам в файловой системе (каждый звук
        соответствует изображению с таким же индексом).
        output_name (str): название выходного видео.
    """

    files = []
    for path_image, path_wav in zip(path_images, path_wavs):
        files.extend([ffmpeg.input(path_image), ffmpeg.input(path_wav)])

    vcodec, acodec = get_codecs(vformat)

    joined = ffmpeg.concat(*files, v=1, a=1).node
    ffmpeg.output(
        joined[0], joined[1], output_name,
        f=vformat, vcodec=vcodec, acodec=acodec
    ).run(overwrite_output=True)



import click

from lecture_automator.compiler import compile_text_md
from lecture_automator.web import run_web


@click.group()
def cli():
    pass


@cli.command()
@click.argument(
    'input_md',
    type=click.STRING,
)
@click.argument(
    'out_path',
    type=click.STRING,
)
@click.option(
    '--vformat',
    type=click.Choice(['mp4', 'webm']),
    default='mp4',
    help=(
        'Формат генерируемого видео.'
    )
)
@click.option(
    '--scale',
    type=click.FLOAT,
    default=1.5,
    help=(
        'Коэффициент масштабирования генерируемого видео, '
        'по умолчанию равен 1.5, что соответствует разрешению 1920x1080,'
        'значение 1 соответствует разрешению 1280x720.'
    )
)
def convert(input_md, out_path, scale, vformat):
    with open(input_md) as file:
        md_text = file.read()

    compile_text_md(
        md_text, out_path=out_path,
        vformat=vformat, scale=scale
    )


@cli.command()
def web():
    run_web()


if __name__ == '__main__':
    cli()



import re
import wave
from typing import List


def divide_text(text: str, max_length: int = 1000) -> List[str]:
    """Разделение текста по точкам таким образом, чтобы каждая часть текста не превышала
    указанный максимум.
    Args:
        text (str): текст для разделения.
        max_length (int, optional): максимальная длина одной части текста. Defaults to
            1000.
    Raises:
        Exception: _description_
    Returns:
        List[str]: список частей текста.
    """

    split_texts = re.split(r'\.', text)
    result_texts = []

    symbols = 0
    last_processed_index_text = 0
    for i, t in enumerate(split_texts):
        if symbols + len(t) >= max_length or i == len(split_texts) - 1:
            if i == 0:
                raise Exception('Текст невозможно разделить!')
            result_texts.append(".".join(split_texts[last_processed_index_text:i]).strip())
            symbols = len(t)
            last_processed_index_text = i
        else:
            symbols += len(t)

    return result_texts


def concat_wavs(input_wavs: List[str], output_wav: str) -> str:
    """Объединение несколько wav-файлов в один.
    Args:
        input_wavs (List[str]): список путей к объединяемым wav-файлам.
        output_wav (str): путь к выходному wav-файлу.
    Returns:
        str: путь к выходному wav-файлу.
    """

    data = []
    for infile in input_wavs:
        with wave.open(infile, 'rb') as w:
            data.append([w.getparams(), w.readframes(w.getnframes())])

    with wave.open(output_wav, 'wb') as output:
        output.setparams(data[0][0])
        for i in range(len(data)):
            output.writeframes(data[i][1])

    return output_wav



import re
from abc import ABC, abstractmethod


class CommandHandler(ABC):
    def __init__(self, handler = None) -> None:
        self._next: CommandHandler = handler

    @abstractmethod
    def handle(self, slide, name_command, arg, start_pos, end_pos, metadata) -> str:
        if self._next is None:
            raise Exception('Неизвестная команда: {}'.format(slide[start_pos:end_pos]))
        return self._next.handle(slide, name_command, arg, start_pos, end_pos)

    @staticmethod
    def process(slide, name_command, arg, start_pos, end_pos, metadata):
        handler = SpeechHandler()
        return handler.handle(slide, name_command, arg, start_pos, end_pos, metadata)


class SpeechHandler(CommandHandler):
    def handle(self, slide, name_command, arg, start_pos, end_pos, metadata) -> str:
        if name_command == 'speech':
            print(slide[end_pos:])
            if re.sub(r'(?m)[ \n\t]+', '', slide[end_pos:]):
                raise Exception(
                    'Управляющая конструкция /speech должна находиться в конце описания слайда!'
                )
            metadata.update(
                {
                    'speech': arg
                }
            )
            return ''
        else:
            return super().handle(slide, name_command, arg, start_pos, end_pos)



# -*- coding: utf-8 -*-

import os
import tempfile

import torch

from lecture_automator.gen_speech.utils import concat_wavs, divide_text
from lecture_automator.settings import get_app_dir

MODEL_URL = 'https://models.silero.ai/models/tts/ru/v3_1_ru.pt'
MODEL_FILENAME = 'v3_1_ru.pt'
SPEAKER = 'xenia'
SAMPLE_RATE = 48000
MAX_SYMBOLS = 1000


def generate_speech(model, text: str, out_path: str, device: str = 'cpu') -> str:
    """Синтез речи.

    Args:
        model (_type_): модель для синтеза речи.
        text (str): текст для синтеза речи.
        out_path (str): полный путь к файлу для сохранения синтезированной речи.
        device (str, optional): устройство для вычислений модели. Defaults to 'cpu'.

    Returns:
        str: путь к синтезированной речи.
    """

    model.to(device)

    model.save_wav(
        text=text,
        speaker=SPEAKER,
        audio_path=out_path,
        sample_rate=SAMPLE_RATE
    )

    return out_path


def get_model(model_name: str):
    """Загрузка модели.

    Args:
        model_name (str): название модели для загрузки.

    Returns:
        _type_: _description_
    """

    model_path = os.path.join(get_app_dir(), model_name)

    if not os.path.exists(model_path):
        torch.hub.download_url_to_file(
            MODEL_URL,
            model_path
        )

    model = torch.package.PackageImporter(model_path).load_pickle(
        "tts_models", "model"
    )

    return model


def text_to_speech(text: str, out_path: str, device: str = 'cpu') -> str:
    """Синтез речи.

    Args:
        text (str): текст для синтеза речи.
        out_path (str): путь для сохранения сгенерированной речи.
        device (str, optional): устройство для вычислений (cuda, cpu и т.д.).
            Defaults to 'cpu'.

    Returns:
        str: название файла формата wav со сгенерированной речью по тексту.
    """

    model = get_model(MODEL_FILENAME)

    if len(text) > MAX_SYMBOLS:
        texts = divide_text(text, max_length=MAX_SYMBOLS)
    else:
        texts = [text]

    with tempfile.TemporaryDirectory() as tmpdirname:
        temp_wavs = []
        for i, t in enumerate(texts):
            temp_out_path = os.path.join(tmpdirname, 'speech_{}'.format(str(i)))
            generate_speech(model, t, temp_out_path)
            temp_wavs.append(temp_out_path)
        concat_wavs(temp_wavs, out_path)

    return out_path


def texts_to_speeches(texts: list, out_dir: str, basename: str = 'Sound') -> list:
    audio_paths = []
    for index, text in enumerate(texts, start=1):
        audio_path = os.path.join(out_dir, '{}_{}.wav'.format(basename, index))
        text_to_speech(
            text, audio_path
        )
        audio_paths.append(audio_path)

    return audio_paths



import os

APP_DIR = os.path.join(os.path.expanduser('~'), '.lecture_automator')

def get_app_dir() -> str:
    if not os.path.exists(APP_DIR):
        os.makedirs(APP_DIR)
    return APP_DIR


def get_cache_dir() -> str:
    cache_path = os.path.join(get_app_dir(), 'cache')
    if not os.path.exists(cache_path):
        os.makedirs(cache_path)
    return cache_path



import os
import subprocess

import streamlit as st

from lecture_automator.compiler import compile_text_md
from lecture_automator.settings import get_cache_dir


def run_web() -> None:
    subprocess.run([
        'streamlit', 'run', os.path.realpath(__file__)
    ])


def main_page() -> None:
    storage_path = get_cache_dir()

    st.markdown('# Lecture Automator')
    text = st.text_area('Текст генерируемой лекции:', height=500)

    if text:
        with st.spinner('Генерируем...'):
            out_path = os.path.join(storage_path, 'Video.webm')
            compile_text_md(
                text, out_path=out_path, vformat='webm')
        st.video(out_path)



if __name__ == '__main__':
    main_page()



import glob
import os
import tempfile

from lecture_automator.gen_speech.gen_speech import texts_to_speeches
from lecture_automator.gen_video.gen_video import generate_video
from lecture_automator.marp_api import generate_marp_slides
from lecture_automator.parser.parser import parse_md


def compile_text_md(input_text_md: str, out_path: str,
                    vformat: str='mp4', scale: int=1) -> None:
    md_data = parse_md(input_text_md)

    with tempfile.TemporaryDirectory() as tmpdirname:
        generate_marp_slides(tmpdirname, md_data['md_text'], scale=scale)
        slide_images = glob.glob(os.path.join(tmpdirname, 'Slide.*'))
        audio_paths = texts_to_speeches(md_data['speech'], tmpdirname)
        generate_video(slide_images, audio_paths, out_path, vformat=vformat)
